@@ -1,7 +1,7 @@
 /*
  * pam_log.c -- PAM system logging
  *
- * $Id: pam_log.c,v 1.4 1999/10/09 04:17:51 morgan Exp $
+ * $Id: pam_log.c,v 1.5 1999/11/08 05:41:05 morgan Exp $
  *
  */
 
@@ -357,10 +357,15 @@ void _pam_system_log(int priority, const char *format, ... )
 
     D(("pam_system_log called"));
 
+    if (format == NULL) {
+	D(("NULL format to _pam_system_log() call"));
+	return;
+    }
+
     va_start(args, format);
 
     eformat = malloc(sizeof(_PAM_SYSTEM_LOG_PREFIX)+strlen(format));
-    if (eformat) {
+    if (eformat != NULL) {
 	strcpy(eformat, _PAM_SYSTEM_LOG_PREFIX);
 	strcpy(eformat + sizeof(_PAM_SYSTEM_LOG_PREFIX) - 1, format);
 	vsyslog(priority, eformat, args);
