@@ -1,7 +1,7 @@
 /* pam_misc.c -- This is random stuff */
 
 /*
- * $Id: pam_misc.c,v 1.3 1998/12/27 04:34:23 morgan Exp $
+ * $Id: pam_misc.c,v 1.4 1999/08/16 03:50:50 morgan Exp $
  */
 
 #include <stdarg.h>
@@ -249,7 +249,7 @@ void _pam_parse_control(int *control_array, char *tok)
 	}
 
 	/* observe action type */
-	for (act=0; act<=-_PAM_ACTION_UNDEF; ++act) {
+	for (act=0; act < (-(_PAM_ACTION_UNDEF)); ++act) {
 	    len = strlen(_pam_token_actions[act]);
 	    if (!strncmp(_pam_token_actions[act], tok, len)) {
 		act *= -1;
