@@ -1,7 +1,7 @@
 /*
  * pam_private.h
  *
- * $Id: pam_private.h,v 1.12 1997/04/05 06:57:37 morgan Exp morgan $
+ * $Id: pam_private.h,v 1.4 1999/06/19 20:51:33 morgan Exp $
  *
  * This is the Linux-PAM Library Private Header. It contains things
  * internal to the Linux-PAM library. Things not needed by either an
@@ -11,15 +11,13 @@
  *
  * Creator: Marc Ewing.
  * Maintained: AGM
- * 
- * $Log: pam_private.h,v $
  */
 
 #ifndef _PAM_PRIVATE_H
 #define _PAM_PRIVATE_H
 
 /* this is not used at the moment --- AGM */
-#define LIBPAM_VERSION 65
+#define LIBPAM_VERSION 67
 
 #include <security/pam_appl.h>
 #include <security/pam_modules.h>
@@ -134,7 +132,6 @@ struct pam_handle {
     char *rhost;
     char *ruser;
     char *tty;
-    struct pam_log_state pam_default_log;      /* for ident etc., log state */
     struct pam_data *data;
     struct pam_environ *env;      /* structure to maintain environment list */
     struct _pam_fail_delay fail_delay;   /* helper function for easy delays */
@@ -216,7 +213,7 @@ typedef void (*voidfunc(void))(void);
 /* The next two in ../modules/_pam_static/pam_static.c */
 
 /* Return pointer to data structure used to define a static module */
-struct pam_module * _pam_open_static_handler(char *path);
+struct pam_module * _pam_open_static_handler(const char *path);
 
 /* Return pointer to function requested from static module */
 
@@ -250,6 +247,9 @@ void _pam_set_default_control(int *control_array, int default_action);
 
 void _pam_parse_control(int *control_array, char *tok);
 
+void _pam_system_log(int priority, const char *format,  ... );
+#define _PAM_SYSTEM_LOG_PREFIX "PAM "
+
 /*
  * XXX - Take care with this. It could confuse the logic of a trailing
  *       else
@@ -257,7 +257,7 @@ void _pam_parse_control(int *control_array, char *tok);
 
 #define IF_NO_PAMH(X,pamh,ERR)                    \
 if ((pamh) == NULL) {                             \
-    pam_system_log(NULL, NULL, LOG_ERR, X ": NULL pam handle passed"); \
+    _pam_system_log(LOG_ERR, X ": NULL pam handle passed"); \
     return ERR;                                   \
 }
 
@@ -265,16 +265,6 @@ if ((pamh) == NULL) {                             \
 
 #define PAM_DEFAULT_PROMPT "Please enter username: "
 
-/*
- * pam_system_log default ident/facility..
- */
-
-#define PAM_LOG_STATE_DEFAULT {      \
-    PAM_LOG_STATE_IDENT,     \
-    PAM_LOG_STATE_OPTION,    \
-    PAM_LOG_STATE_FACILITY   \
-}
-
 /*
  * include some helpful macros
  */
