@@ -29,11 +29,12 @@
  * S.A.G. in the section on the cracklib module.
  */
 
+#define _GNU_SOURCE
+#define _BSD_SOURCE
+
 #include <stdio.h>
 #ifdef NEED_CRYPT_HEADER
 # include <crypt.h>
-#else
-# define _BSD_SOURCE
 #endif
 #include <unistd.h>
 #include <stdlib.h>
