@@ -1,7 +1,7 @@
 /* pam_ftp module */
 
 /*
- * $Id: pam_ftp.c,v 1.2 1999/10/09 05:10:21 morgan Exp $
+ * $Id: pam_ftp.c,v 1.3 1999/11/08 05:46:55 morgan Exp $
  *
  * Written by Andrew Morgan <morgan@linux.kernel.org> 1996/3/11
  *
@@ -14,11 +14,10 @@
 /* the following is a password that "can't be correct" */
 #define BLOCK_PASSWORD "\177BAD PASSWPRD\177"
 
-#ifdef linux
-# define _GNU_SOURCE
-# include <features.h>
-#endif
+#define _GNU_SOURCE
+#define _BSD_SOURCE
 
+#include <features.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -203,16 +202,22 @@ int pam_sm_authenticate(pam_handle_t *pamh,int flags,int argc
 	char *prompt=NULL;
 	int i=0;
 
-	mesg[i] = &msg[i];
-	msg[i].msg_style = PAM_PROMPT_ECHO_OFF;
-	if (anon) {
-	    prompt = malloc(sizeof(PLEASE_ENTER_PASSWORD) + strlen(user));
-	    sprintf(prompt, PLEASE_ENTER_PASSWORD, user);
-	    msg[i].msg = prompt;
+	if (!anon) {
+	    prompt = malloc(strlen(PLEASE_ENTER_PASSWORD) + strlen(user));
+	    if (prompt == NULL) {
+		D(("out of memory!?"));
+		return PAM_BUF_ERR;
+	    } else {
+		sprintf(prompt, PLEASE_ENTER_PASSWORD, user);
+		msg[i].msg = prompt;
+	    }
 	} else {
 	    msg[i].msg = GUEST_LOGIN_PROMPT;
 	}
 
+	msg[i].msg_style = PAM_PROMPT_ECHO_OFF;
+	mesg[i] = &msg[i];
+
 	retval = converse(pamh, ++i, mesg, &resp);
 	if (prompt) {
 	    _pam_overwrite(prompt);
