@@ -470,8 +470,20 @@ static int parse_config_file(const char *uname, int ctrl)
                     process_limit(LIMITS_DEF_GROUP, ltype, item, value, ctrl);
             } else if (strcmp(domain, "*") == 0)
                 process_limit(LIMITS_DEF_DEFAULT, ltype, item, value, ctrl);
-        } else
+	} else if (i == 2 && ltype[0] == '-') { /* Probably a no-limit line */
+	    if (strcmp(uname, domain) == 0) {
+		_pam_log(LOG_DEBUG, "no limits for '%s'", uname);
+		fclose(fil);
+		return PAM_IGNORE;
+	    } else if (domain[0] == '@' && is_on_group(uname, domain+1)) {
+		_pam_log(LOG_DEBUG, "no limits for '%s' in group '%s'",
+			 uname, domain+1);
+		fclose(fil);
+		return PAM_IGNORE;
+	    }
+        } else {
             _pam_log(LOG_DEBUG,"invalid line '%s'", buf);
+	}
     }
     fclose(fil);
     return PAM_SUCCESS;    
