@@ -1,11 +1,12 @@
 /* 
- * $Id: support.c,v 1.2 1999/07/05 05:53:10 morgan Exp $
+ * $Id: support.c,v 1.3 1999/10/09 05:07:32 morgan Exp $
  *
  * Copyright information at end of file.
  */
 
+#define _BSD_SOURCE
+
 #include <stdlib.h>
-#define __USE_BSD
 #include <unistd.h>
 #include <stdarg.h>
 #include <string.h>
@@ -24,9 +25,6 @@
 extern char *crypt(const char *key, const char *salt);
 extern char *bigcrypt(const char *key, const char *salt);
 
-/* How many passwords should we remember? */
-int remember = -1;
-
 /* syslogging function for errors and other information */
 
 void _log_err(int err, const char *format,...)
@@ -128,7 +126,7 @@ char *PAM_getlogin(void)
  * set the control flags for the UNIX module.
  */
 
-int _set_ctrl(int flags, int argc, const char **argv)
+int _set_ctrl(int flags, int *remember, int argc, const char **argv)
 {
 	unsigned int ctrl;
 
@@ -178,12 +176,14 @@ int _set_ctrl(int flags, int argc, const char **argv)
 			ctrl &= unix_args[j].mask;	/* for turning things off */
 			ctrl |= unix_args[j].flag;	/* for turning things on  */
 
-			if (j == UNIX_REMEMBER_PASSWD) {
-				remember = strtol(*argv + 9, NULL, 10);
-				if ((remember == LONG_MIN) || (remember == LONG_MAX))
-					remember = -1;
-				if (remember > 400)
-					remember = 400;
+			if (remember != NULL) {
+				if (j == UNIX_REMEMBER_PASSWD) {
+					*remember = strtol(*argv + 9, NULL, 10);
+					if ((*remember == LONG_MIN) || (*remember == LONG_MAX))
+						*remember = -1;
+					if (*remember > 400)
+						*remember = 400;
+				}
 			}
 		}
 
