@@ -24,7 +24,7 @@ static int converse(pam_handle_t *pamh,
 		    struct pam_response **response)
 {
     int retval;
-    struct pam_conv *conv;
+    const struct pam_conv *conv;
 
     retval = pam_get_item(pamh, PAM_CONV, (const void **) &conv ) ;
     if (retval == PAM_SUCCESS)
